using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;

namespace MapApp.Model
{
    [Serializable]
    public class MapLocationCollection : BindingList<MapLocation>
    {
        private MapLocation _mostRecentlyRemoved;

        public MapLocation MostRecentlyRemoved
        {
            get { return _mostRecentlyRemoved; }
        }

        public void Save(Stream outStream) {
            BinaryFormatter bf = new BinaryFormatter();
            bf.Serialize(outStream, this);
        }

        public static MapLocationCollection Load(Stream inStream)
        {
            BinaryFormatter bf = new BinaryFormatter();
            return (MapLocationCollection)bf.Deserialize(inStream);
        }

        public MapLocation Find(int id)
        {
            // For now, just do a linear search...
            foreach (MapLocation loc in this)
            {
                if (loc.Id == id)
                {
                    return loc;
                    //break;
                }
            }
            return null;
        }

        protected override void RemoveItem(int index)
        {
            _mostRecentlyRemoved = this[index];
            base.RemoveItem(index);
        }



    }

    [Serializable]
    public class MapLocation : INotifyPropertyChanged
    {
        [NonSerialized]
        int _id = 0;
        double _lat = 0.0;
        double _lng = 0.0;
        int _zoomLevel = 0;
        string _label = "";
        string _description = "";
        string _locationString = "";
        Uri _url = null;


        #region Constructors

        public MapLocation(string locationString, double lat, double lng)
        {
            this.LocationString = locationString;
            this.Lat = lat;
            this.Lng = lng;
        }

        public MapLocation(string locationString, double lat, double lng, int zoomLevel)
        {
            this.LocationString = locationString;
            this.Lat = lat;
            this.Lng = lng;
            this.ZoomLevel = zoomLevel;
        }

        #endregion // Constructors

        #region Properties

        
        public int Id
        {
            get { return _id; }
            set
            {
                if (_id != value)
                {
                    _id = value;
                    NotifyPropertyChanged("Id");
                }
            }
        }

        public string Label
        {
            get { return _label; }
            set
            {
                if (_label != value)
                {
                    _label = value;
                    NotifyPropertyChanged("Label");
                }
            }
        }

        public string Description
        {
            get { return _description; }
            set
            {
                if (_description != value)
                {
                    _description = value;
                    NotifyPropertyChanged("Description");
                }
            }
        }

        public Uri Url
        {
            get { return _url; }
            set
            {
                if ((_url != null && !_url.Equals(value)) ||
                    (value != null) && !value.Equals(_url))
                {
                    _url = value;
                    NotifyPropertyChanged("Url");
                }
            }
        }

        public String LocationString
        {
            get { return _locationString; }
            set
            {
                if (_locationString != value)
                {
                    _locationString = value;
                    NotifyPropertyChanged("LocationString");
                }
            }
        }

        public double Lat
        {
            get { return _lat; }
            set
            {
                if (_lat != value)
                {
                    _lat = value;
                    NotifyPropertyChanged("Lat");
                }
            }
        }

        public double Lng
        {
            get { return _lng; }
            set
            {
                if (_lng != value)
                {
                    _lng = value;
                    NotifyPropertyChanged("Lng");
                }

            }
        }

        public int ZoomLevel
        {
            get { return _zoomLevel; }
            set
            {
                if (_zoomLevel != value)
                {
                    _zoomLevel = value;
                    NotifyPropertyChanged("ZoomLevel");
                }
            }
        }

        #endregion // Properties


        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(String propName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propName));
            }
        }

        #endregion
    }
}
